#!/usr/bin/python
# -*- coding: UTF-8 -*-


"""
Author: Kristian K Ullrich
date: April 2023
email: ullrich@evolbio.mpg.de
License: GPL-3
"""


import os
import wget
import scanpy as sc


def ensembl105(datapath='.'):
    """
    OrthoFinder results for all translated coding sequences (CDS) from Ensembl release-105
    (keeping only longest isoforms)
    and Xtropicalisv9.0.Named.primaryTrs.pep.fa from www.xenbase.org.

    All files can be obtained from here:
    https://doi.org/10.5281/zenodo.7242263

    :param datapath: Path to safe dataset.
    :return: Path to Orthogroups.GeneCount file, OrthoGroups file and species list file.

    :type datapath: str
    :rtype: list of str

    Example
    -------
    >>> from orthomap import datasets
    >>> datasets.ensembl105(datapath='.')
    """
    if not os.path.exists(datapath):
        print('datapath does not exist, is created now')
        os.makedirs(name=datapath)
    oc_filename = os.path.join(datapath,
                               'ensembl_105_orthofinder_Orthogroups.GeneCount.tsv.zip')
    og_filename = os.path.join(datapath,
                               'ensembl_105_orthofinder_Orthogroups.tsv.zip')
    sl_filename = os.path.join(datapath,
                               'ensembl_105_orthofinder_species_list.tsv')
    oc_url = 'https://zenodo.org/record/7796253/files/ensembl_105_orthofinder_Orthogroups.GeneCount.tsv.zip'
    og_url = 'https://zenodo.org/record/7796253/files/ensembl_105_orthofinder_Orthogroups.tsv.zip'
    sl_url = 'https://zenodo.org/record/7796253/files/ensembl_105_orthofinder_species_list.tsv'

    wget.download(url=oc_url,
                  out=datapath)
    wget.download(url=og_url,
                  out=datapath)
    wget.download(url=sl_url,
                  out=datapath)
    return [oc_filename,
            og_filename,
            sl_filename]


def zebrafish_gtf(datapath='.'):
    """
    Download GTF for species Danio rerio from ensembl release-105
    https://ftp.ensembl.org/pub/release-105/gtf/danio_rerio/

    :param datapath: Path to safe dataset.
    :return: Path to GTF file.

    :type datapath: str
    :rtype: str

    Example
    -------
    >>> from orthomap import datasets
    >>> datasets.zebrafish_gtf(datapath='.')
    """
    if not os.path.exists(datapath):
        print('datapath does not exist, is created now')
        os.makedirs(name=datapath)
    zebrafish_gtf_filename = os.path.join(datapath,
                                          'Danio_rerio.GRCz11.105.gtf.gz')
    zebrafish_gtf_url = 'https://ftp.ensembl.org/pub/release-105/gtf/danio_rerio/Danio_rerio.GRCz11.105.gtf.gz'
    wget.download(url=zebrafish_gtf_url,
                  out=datapath)
    return zebrafish_gtf_filename


def mouse_gtf(datapath='.'):
    """
    Download GTF for species Mus musculus from ensembl release-105
    https://ftp.ensembl.org/pub/release-105/gtf/mus_musculus/

    :param datapath: Path to safe dataset.
    :return: Path to GTF file.

    :type datapath: str
    :rtype: str

    Example
    -------
    >>> from orthomap import datasets
    >>> datasets.mouse_gtf(datapath='.')
    """
    if not os.path.exists(datapath):
        print('datapath does not exist, is created now')
        os.makedirs(name=datapath)
    mouse_gtf_filename = os.path.join(datapath,
                                      'Mus_musculus.GRCm39.105.gtf.gz')
    mouse_gtf_url = 'https://ftp.ensembl.org/pub/release-105/gtf/mus_musculus/Mus_musculus.GRCm39.105.gtf.gz'
    wget.download(url=mouse_gtf_url,
                  out=datapath)
    return mouse_gtf_filename


def sun21_orthomap(datapath='.'):
    """
    Pre-calculated orthomap for Caenorhabditis elegans from:

    Sun, S., Rödelsperger, C. and Sommer, R.J., 2021.
    Single worm transcriptomics identifies a developmental core network of oscillating genes with deep
    conservation across nematodes.
    Genome research, 31(9), pp.1590-1601.

    :param datapath: Path to safe dataset.
    :return: Path to Orthomap file.

    :type datapath: str
    :rtype: str

    Example
    -------
    >>> from orthomap import datasets
    >>> datasets.sun21_orthomap(datapath='.')
    """
    if not os.path.exists(datapath):
        print('datapath does not exist, is created now')
        os.makedirs(name=datapath)
    sun21_orthomap_filename = os.path.join(datapath,
                                           'Sun2021_Orthomap.tsv')
    sun21_orthomap_url = 'https://zenodo.org/record/7783163/files/Sun2021_Orthomap.tsv'
    wget.download(url=sun21_orthomap_url,
                  out=datapath)
    return sun21_orthomap_filename


def ma21_fst(datapath='.'):
    """
    Pre-calculated TajimaD, NormalizedPi, FayWu and Fst for Caenorhabditis elegans from:

    Ma, F., Lau, C.Y. and Zheng, C., 2021.
    Large genetic diversity and strong positive selection in F-box and GPCR genes among the wild isolates of
    Caenorhabditis elegans.
    Genome Biology and Evolution, 13(5), p.evab048.

    :param datapath: Path to safe dataset.
    :return: Path to diversity file.

    :type datapath: str
    :rtype: str

    Example
    -------
    >>> from orthomap import datasets
    >>> datasets.ma21_orthomap(datapath='.')
    """
    if not os.path.exists(datapath):
        print('datapath does not exist, is created now')
        os.makedirs(name=datapath)
    ma21_orthomap_filename = os.path.join(datapath,
                                          'Ma2021_Fst.tsv')
    ma21_orthomap_url = 'https://zenodo.org/record/7802177/files/Ma2021_Fst.tsv'
    wget.download(url=ma21_orthomap_url,
                  out=datapath)
    return ma21_orthomap_filename


def cazet22_orthomap(datapath='.'):
    """
    Pre-calculated orthomap for Hydra vulgaris from:

    Cazet, Jack, Stefan Siebert, Hannah Morris Little, Philip Bertemes, Abby S. Primack, Peter Ladurner,
    Matthias Achrainer et al. (2022)
    New Hydra genomes reveal conserved principles of hydrozoan transcriptional regulation.,
    bioRxiv, 2022.06.21.496857.

    :param datapath: Path to safe dataset.
    :return: Path to Orthomap file.

    :type datapath: str
    :rtype: str

    Example
    -------
    >>> from orthomap import datasets
    >>> datasets.cazet22_orthomap(datapath='.')
    """
    if not os.path.exists(datapath):
        print('datapath does not exist, is created now')
        os.makedirs(name=datapath)
    cazet22_orthomap_filename = os.path.join(datapath,
                                             'Cazet2022_Orthomap.tsv')
    cazet22_orthomap_url = 'https://zenodo.org/record/7404798/files/Cazet2022_Orthomap.tsv'
    wget.download(url=cazet22_orthomap_url,
                  out=datapath)
    return cazet22_orthomap_filename


def packer19(datapath='.'):
    """
    scRNA count data for Caenorhabditis elegans from:

    Packer, J.S., Zhu, Q., Huynh, C., Sivaramakrishnan, P., Preston, E., Dueck, H., Stefanik, D.,
    Tan, K., Trapnell, C., Kim, J. and Waterston, R.H., 2019.
    A lineage-resolved molecular atlas of C. elegans embryogenesis at single-cell resolution.
    Science, 365(6459), p.eaax1971.

    All files can be obtained from here:
    https://doi.org/10.5281/zenodo.7245547

    :param datapath: Path to safe dataset.
    :return: AnnData object.

    :type datapath: str
    :rtype: AnnData

    Example
    -------
    >>> from orthomap import datasets
    >>> datasets.packer19(datapath='.')
    """
    if not os.path.exists(datapath):
        print('datapath does not exist, is created now')
        os.makedirs(name=datapath)
    packer19_filename = os.path.join(datapath,
                                     'GSE126954.h5ad')
    packer19_url = 'https://zenodo.org/record/7496490/files/GSE126954.h5ad'
    wget.download(url=packer19_url,
                  out=datapath)
    adata = sc.read(packer19_filename)
    return adata


def packer19_small(datapath='.'):
    """
    scRNA count data for Caenorhabditis elegans from:

    Packer, J.S., Zhu, Q., Huynh, C., Sivaramakrishnan, P., Preston, E., Dueck, H., Stefanik, D.,
    Tan, K., Trapnell, C., Kim, J. and Waterston, R.H., 2019.
    A lineage-resolved molecular atlas of C. elegans embryogenesis at single-cell resolution.
    Science, 365(6459), p.eaax1971.

    All files can be obtained from here:
    https://doi.org/10.5281/zenodo.7245547

    :param datapath: Path to safe dataset.
    :return: AnnData object.

    :type datapath: str
    :rtype: AnnData

    Example
    -------
    >>> from orthomap import datasets
    >>> datasets.packer19_small(datapath='.')
    """
    if not os.path.exists(datapath):
        print('datapath does not exist, is created now')
        os.makedirs(name=datapath)
    packer19_small_filename = os.path.join(datapath,
                                           'packer19_small.h5ad')
    packer19_small_url = 'https://zenodo.org/record/7496490/files/packer19_small.h5ad'
    wget.download(url=packer19_small_url,
                  out=datapath)
    adata = sc.read(packer19_small_filename)
    return adata


def cazet22(datapath='.'):
    """
    scRNA count data for Hydra vulgaris from:

    Cazet, Jack, Stefan Siebert, Hannah Morris Little, Philip Bertemes, Abby S. Primack, Peter Ladurner,
    Matthias Achrainer et al. (2022)
    New Hydra genomes reveal conserved principles of hydrozoan transcriptional regulation.,
    bioRxiv, 2022.06.21.496857.

    All files can be obtained from here:
    https://doi.org/10.5281/zenodo.7366178

    :param datapath: Path to safe dataset.
    :return: AnnData object.

    :type datapath: str
    :rtype: AnnData

    Example
    -------
    >>> from orthomap import datasets
    >>> datasets.cazet22(datapath='.')
    """
    if not os.path.exists(datapath):
        print('datapath does not exist, is created now')
        os.makedirs(name=datapath)
    cazet22_filename = os.path.join(datapath,
                                    'aepAtlasNonDub.h5ad')
    cazet22_url = 'https://zenodo.org/record/7369647/files/aepAtlasNonDub.h5ad'
    wget.download(url=cazet22_url,
                  out=datapath)
    adata = sc.read(cazet22_filename)
    return adata


def qiu22_zebrafish(datapath='.'):
    """
    combined scRNA count data for Danio rerio from:

    Qiu, C., Cao, J., Martin, B.K., Li, T., Welsh, I.C., Srivatsan, S., Huang, X., Calderon,
    D., Noble, W.S., Disteche, C.M. and Murray, S.A., 2022.
    Systematic reconstruction of cellular trajectories across mouse embryogenesis.
    Nature genetics, 54(3), pp.328-341.

    original scRNA count data from:

    Farrell, J.A., Wang, Y., Riesenfeld, S.J., Shekhar, K., Regev, A. and Schier, A.F., 2018.
    Single-cell reconstruction of developmental trajectories during zebrafish embryogenesis.
    Science, 360(6392), p.eaar3131.

    Wagner, D.E., Weinreb, C., Collins, Z.M., Briggs, J.A., Megason, S.G. and Klein, A.M., 2018.
    Single-cell mapping of gene expression landscapes and lineage in the zebrafish embryo.
    Science, 360(6392), pp.981-987.

    All files can be obtained from here:
    https://doi.org/10.5281/zenodo.7243602

    :param datapath: Path to safe dataset.
    :return: AnnData object.

    :type datapath: str
    :rtype: AnnData

    Example
    -------
    >>> from orthomap import datasets
    >>> datasets.qiu22_zebrafish(datapath='.')
    """
    if not os.path.exists(datapath):
        print('datapath does not exist, is created now')
        os.makedirs(name=datapath)
    qiu22_zebrafish_filename = os.path.join(datapath,
                                            'zebrafish_data.h5ad')
    qiu22_zebrafish_url = 'https://zenodo.org/record/7243603/files/zebrafish_data.h5ad'
    wget.download(url=qiu22_zebrafish_url,
                  out=datapath)
    adata = sc.read(qiu22_zebrafish_filename)
    return adata


def qiu22_frog(datapath='.'):
    """
    combined scRNA count data for Xenopus tropicalis from:

    Qiu, C., Cao, J., Martin, B.K., Li, T., Welsh, I.C., Srivatsan, S., Huang, X., Calderon,
    D., Noble, W.S., Disteche, C.M. and Murray, S.A., 2022.
    Systematic reconstruction of cellular trajectories across mouse embryogenesis.
    Nature genetics, 54(3), pp.328-341.

    original scRNA count data from:

    Briggs, J.A., Weinreb, C., Wagner, D.E., Megason, S., Peshkin, L., Kirschner, M.W. and Klein, A.M., 2018.
    The dynamics of gene expression in vertebrate embryogenesis at single-cell resolution.
    Science, 360(6392), p.eaar5780.

    All files can be obtained from here:
    https://doi.org/10.5281/zenodo.7244440

    :param datapath: Path to safe dataset.
    :return: AnnData object.

    :type datapath: str
    :rtype: AnnData

    Example
    -------
    >>> from orthomap import datasets
    >>> datasets.qiu22_frog(datapath='.')
    """
    if not os.path.exists(datapath):
        print('datapath does not exist, is created now')
        os.makedirs(name=datapath)
    qiu22_frog_filename = os.path.join(datapath,
                                       'frog_data.h5ad')
    qiu22_frog_url = 'https://zenodo.org/record/7244441/files/frog_data.h5ad'
    wget.download(url=qiu22_frog_url,
                  out=datapath)
    adata = sc.read(qiu22_frog_filename)
    return adata


def qiu22_mouse(datapath='.'):
    """
    combined scRNA count data for Mus musculus from:

    Qiu, C., Cao, J., Martin, B.K., Li, T., Welsh, I.C., Srivatsan, S., Huang, X., Calderon,
    D., Noble, W.S., Disteche, C.M. and Murray, S.A., 2022.
    Systematic reconstruction of cellular trajectories across mouse embryogenesis.
    Nature genetics, 54(3), pp.328-341.

    original scRNA count data from:

    Mohammed, H., Hernando-Herraez, I., Savino, A., Scialdone, A., Macaulay, I., Mulas, C., Chandra, T.,
    Voet, T., Dean, W., Nichols, J. and Marioni, J.C., 2017.
    Single-cell landscape of transcriptional heterogeneity and cell fate decisions during mouse early gastrulation.
    Cell reports, 20(5), pp.1215-1228.

    Cheng, S., Pei, Y., He, L., Peng, G., Reinius, B., Tam, P.P., Jing, N. and Deng, Q., 2019.
    Single-cell RNA-seq reveals cellular heterogeneity of pluripotency transition and X chromosome dynamics
    during early mouse development.
    Cell reports, 26(10), pp.2593-2607.

    Pijuan-Sala, B., Griffiths, J.A., Guibentif, C., Hiscock, T.W., Jawaid, W., Calero-Nieto, F.J.,
    Mulas, C., Ibarra-Soria, X., Tyser, R.C., Ho, D.L.L. and Reik, W., 2019.
    A single-cell molecular map of mouse gastrulation and early organogenesis.
    Nature, 566(7745), pp.490-495.

    Cao, J., Spielmann, M., Qiu, X., Huang, X., Ibrahim, D.M., Hill, A.J., Zhang, F., Mundlos,
    S., Christiansen, L., Steemers, F.J. and Trapnell, C., 2019.
    The single-cell transcriptional landscape of mammalian organogenesis.
    Nature, 566(7745), pp.496-502.

    All files can be obtained from here:
    https://doi.org/10.5281/zenodo.7244567

    :param datapath: Path to safe dataset.
    :return: AnnData object.

    :type datapath: str
    :rtype: AnnData

    Example
    -------
    >>> from orthomap import datasets
    >>> datasets.qiu22_mouse(datapath='.')
    """
    if not os.path.exists(datapath):
        print('datapath does not exist, is created now')
        os.makedirs(name=datapath)
    qiu22_mouse_filename = os.path.join(datapath,
                                        'mouse_data.h5ad')
    qiu22_mouse_url = 'https://zenodo.org/record/7244568/files/mouse_data.h5ad'
    wget.download(url=qiu22_mouse_url,
                  out=datapath)
    adata = sc.read(qiu22_mouse_filename)
    return adata


def mytai_example(datapath='.'):
    """
    expression count data for Arabidopsis thaliana from:

    Drost, H.G., Janitza, P., Grosse, I. and Quint, M., 2017.
    Cross-kingdom comparison of the developmental hourglass.
    Current Opinion in Genetics & Development, 45, pp.69-75.

    All files can be obtained from here:
    https://doi.org/10.5281/zenodo.7242263

    :param datapath: Path to safe dataset.
    :return: AnnData object.

    :type datapath: str
    :rtype: AnnData

    Example
    -------
    >>> from orthomap import datasets
    >>> datasets.mytai_example(datapath='.')
    """
    if not os.path.exists(datapath):
        print('datapath does not exist, is created now')
        os.makedirs(name=datapath)
    mytai_example_filename = os.path.join(datapath,
                                          'PhyloExpressionSetExample.h5ad')
    mytai_example_url = 'https://zenodo.org/record/7783163/files/PhyloExpressionSetExample.h5ad'
    wget.download(url=mytai_example_url,
                  out=datapath)
    adata = sc.read(mytai_example_filename)
    return adata



#!/usr/bin/python
# -*- coding: UTF-8 -*-


"""
Author: Kristian K Ullrich
date: April 2023
email: ullrich@evolbio.mpg.de
License: GPL-3
"""

import os
import sys
import argparse
import pandas as pd
import numpy as np
from orthomap import of2orthomap, qlin
from ete3 import NCBITaxa


def define_parser():
    """
    A helper function for using `plaza2orthomap.py` via the terminal.

    :return: An argparse.ArgumentParser.

    :rtype: argparse.ArgumentParser
    """
    plaza2orthomap_example = '''example:

    # using Orthologous gene family 
    $ plaza2orthomap -qt 3702 -sl species_information.csv -og genefamily_data.ORTHOFAM.csv -out 3702.orthofam.orthomap
    
    # using Homologous gene family 
    $ plaza2orthomap -qt 3702 -sl species_information.csv -og genefamily_data.HOMFAM.csv -out 3702.homfam.orthomap
    '''
    parser = argparse.ArgumentParser(
        prog='plaza2orthomap',
        usage='%(prog)s [options] [<arguments>...]',
        description='extract orthomap from PLAZA gene family data for query species',
        epilog=plaza2orthomap_example,
        formatter_class=argparse.RawDescriptionHelpFormatter)
    add_argparse_args(parser=parser)
    return parser


def add_argparse_args(parser: argparse.ArgumentParser):
    """
    This function attaches individual argument specifications to the parser.

    :param parser: An argparse.ArgumentParser.

    :type parser: argparse.ArgumentParser
    """
    parser.add_argument('-qt',
                        help='query species taxID (e.g. use <orthomap qlin -h> to get taxID)')
    parser.add_argument('-sl',
                        help='specify PLAZA species information file <species_information.csv>')
    parser.add_argument('-og',
                        help='specify PLAZA gene family file <genefamily_data.ORTHOFAM.csv> or '
                             'genefamily_data.HOMFAM.csv')
    parser.add_argument('-out',
                        help='specify output file <orthomap.tsv> (default: orthomap.tsv)',
                        default='orthomap.tsv')
    parser.add_argument('-overwrite',
                        help='specify if existing output file should be overwritten (default: True)',
                        default=True,
                        type=bool)


def _get_species_tax_id(species_name_list, species_list):
    """
    A helper function to map PLAZA species short name and species taxID.

    :param species_name_list: List of PLAZA species short names.
    :param species_list: DataFrame with PLAZA species information.
    :return: List of species taxID.

    :type species_name_list: list
    :type species_list: pandas.DataFrame
    :rtype: list
    """
    species_taxid_list = []
    for species_name in species_name_list:
        species_taxid = list(species_list['tax_id'][species_list['species'] == species_name])
        if len(species_taxid) > 0:
            species_taxid_list.append(species_taxid[0])
    return species_taxid_list


def get_plaza_orthomap(qt,
                       sl,
                       og,
                       out=None,
                       quiet=False,
                       continuity=True,
                       overwrite=True):
    """
    This function return an orthomap for a given query species and PLAZA gene family data.

    :param qt: Query species taxID.
    :param sl: Path to PLAZA species information <species_information.csv> file.
    :param og: Path to PLAZA gene families <genefamily_data.ORTHOFAM.csv> file.
    :param out: Path to output file.
    :param quiet: Specify if output should be quiet.
    :param continuity: Specify if continuity score should be calculated.
    :param overwrite: Specify if output should be overwritten.
    :return: A list of results such as:
             orthomap, species_list, youngest_common_counts

    :type qt: str
    :type sl: str
    :type og: str
    :type out: str
    :type quiet: bool
    :type continuity: bool
    :type overwrite: bool
    :rtype: list

    Example
    -------
    >>>
    """
    outhandle = None
    og_continuity_score = None
    ncbi = NCBITaxa()
    qname,\
        qtid,\
        qlineage,\
        qlineagenames_dict,\
        qlineagezip,\
        qlineagenames,\
        qlineagerev,\
        qk = qlin.get_qlin(qt=qt,
                           quiet=True)
    query_lineage_topo = qlin.get_lineage_topo(qt)
    species_list = pd.read_csv(sl, sep='\t', header=None, comment='#')
    species_list.columns = ['species', 'common_name', 'tax_id', 'source', 'data_provider', 'pubmed_id']
    qt_species = list(species_list['species'][species_list['tax_id'] == int(qt)])
    if len(qt_species) == 0:
        print('\nError <-qt>: query species taxID not in PLAZA results, please check taxID.')
        sys.exit()
    ogs = pd.DataFrame(pd.read_csv(og, sep='\t', header=None, comment='#'))
    ogs.columns = ['gf_id', 'species', 'gene_id']
    ogs_grouped = ogs.groupby('gf_id')['species'].apply(set).apply(list).apply(_get_species_tax_id,
                                                                               species_list=species_list)
    ogs_grouped_qt = pd.DataFrame(ogs_grouped[ogs_grouped.apply(lambda x: int(qt) in x)])
    ogs_qt = ogs[ogs['gf_id'].isin(ogs_grouped_qt.index)]
    ogs_qt_red = ogs_qt[ogs_qt['species'].isin(qt_species)]
    ogs_qt_red_grouped = ogs_qt_red.groupby('gf_id')['gene_id'].apply(list)
    ogs_grouped_qt['gene_id'] = ogs_qt_red_grouped
    ogs_grouped_qt_species = np.sort(list(set([x[0] for x in ogs_grouped_qt['species'].to_dict().values()])))
    ogs_grouped_qt_species_names = [qlin.get_qlin(qt=x,
                                                  quiet=True)[0] for x in ogs_grouped_qt_species]
    species_list_df = pd.DataFrame(ogs_grouped_qt_species_names,
                                   columns=['species'])
    species_list_df['taxID'] = ogs_grouped_qt_species
    species_list_df['lineage'] = species_list_df.apply(lambda x: ncbi.get_lineage(x[1]),
                                                       axis=1)
    species_list_df['youngest_common'] = [qlin.get_youngest_common(qlineage,
                                                                   x) for x in species_list_df.lineage]
    species_list_df['youngest_name'] = [list(x.values())[0] for x in [ncbi.get_taxid_translator([x])
                                                                      for x in list(species_list_df.youngest_common)]]
    if not quiet:
        print(qname)
        print(qt)
        print(species_list_df)
    youngest_common_counts_df = of2orthomap.get_youngest_common_counts(qlineage,
                                                                       species_list_df)
    for node in query_lineage_topo.traverse('postorder'):
        nsplit = node.name.split('/')
        if len(nsplit) == 3:
            node.add_feature('species_count',
                             list(youngest_common_counts_df[youngest_common_counts_df.PStaxID.isin(
                                 [int(nsplit[1])])].counts)[0])
    og_dict = {}
    continuity_dict = {}
    for og in ogs_grouped_qt.index:
        og_hits = np.sort(
            list(set(list(ogs_grouped_qt[ogs_grouped_qt.index.isin([og])]['species'].to_dict().values())[0])))
        # get list of the youngest common between query and all other species
        og_hits_youngest_common = list(species_list_df.youngest_common[
                                           [x for x, y in enumerate(species_list_df.taxID)
                                            if y in og_hits]])
        # evaluate all youngest common nodes to retain the oldest of them and assign as the orthogroup
        # ancestral state (gene age)
        if len(og_hits_youngest_common) > 0:
            og_oldest_common = qlin.get_oldest_common(qlineage,
                                                      og_hits_youngest_common)
            og_dict[og] = og_oldest_common
            if continuity:
                continuity_dict[og] = \
                    of2orthomap.get_youngest_common_counts(qlineage,
                                                           pd.DataFrame(og_hits_youngest_common,
                                                                        columns=['youngest_common'])).counts
    if continuity:
        youngest_common_counts_df = youngest_common_counts_df.join(pd.DataFrame.from_dict(continuity_dict))
    omap = []
    if out:
        if os.path.exists(out) and not overwrite:
            print('\nError <-overwrite>: output file exists, please set to True if it should be overwritten\n')
            sys.exit()
        outhandle = open(out, 'w')
        if continuity:
            outhandle.write('seqID\tOrthogroup\tPSnum\tPStaxID\tPSname\tPScontinuity\n')
        else:
            outhandle.write('seqID\tOrthogroup\tPSnum\tPStaxID\tPSname\n')
    for og in ogs_grouped_qt.index:
        og_tmp = ogs_grouped_qt[ogs_grouped_qt.index.isin([og])]
        og_ps = qlineagenames[qlineagenames['PStaxID'] ==
                              str(og_dict[og])].values.tolist()[0]
        og_ps_join = '\t'.join(og_ps)
        if continuity:
            og_continuity_score = of2orthomap.get_continuity_score(og,
                                                                   youngest_common_counts_df)
            if out:
                if continuity:
                    [outhandle.write(x.replace(' ', '') + '\t' + og + '\t' + og_ps_join + '\t' +
                                     str(og_continuity_score) + '\n') for x in list(og_tmp['gene_id'])[0]]
                else:
                    [outhandle.write(x.replace(' ', '') + '\t' + og + '\t' + og_ps_join + '\n')
                     for x in list(og_tmp['gene_id'])[0]]
        if continuity:
            omap += [[x.replace(' ', ''), og, og_ps[0], og_ps[1], og_ps[2], og_continuity_score]
                     for x in list(og_tmp['gene_id'])[0]]
        else:
            omap += [[x.replace(' ', ''), og, og_ps[0], og_ps[1], og_ps[2]]
                     for x in list(og_tmp['gene_id'])[0]]
    if out:
        outhandle.close()
    omap_df = pd.DataFrame(omap)
    if continuity:
        omap_df.columns = ['seqID',
                           'Orthogroup',
                           'PSnum',
                           'PStaxID',
                           'PSname',
                           'PScontinuity']
    else:
        omap_df.columns = ['seqID',
                           'Orthogroup',
                           'PSnum',
                           'PStaxID',
                           'PSname']
    omap_df['PSnum'] = [int(x) for x in list(omap_df['PSnum'])]
    return [omap_df,
            species_list_df,
            youngest_common_counts_df]


def main():
    """
    The main function that is being called when `plaza2orthomap` is used via the terminal.
    """
    parser = define_parser()
    args = parser.parse_args()
    print(args)
    if not args.qt:
        parser.print_help()
        print('\nError <-qt>: Please specify query species taxID')
        sys.exit()
    if not args.sl:
        parser.print_help()
        print('\nError <-sl>: Please specify PLAZA species information file <species_information.csv>')
        sys.exit()
    if not args.og:
        parser.print_help()
        print('\nError <-og>: Please specify PLAZA gene family file <genefamily_data.ORTHOFAM.csv> or '
              '<genefamily_data.HOMFAM.csv>')
        sys.exit()
    get_plaza_orthomap(qt=args.qt,
                       sl=args.sl,
                       og=args.og,
                       out=args.out,
                       overwrite=args.overwrite)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: UTF-8 -*-

import orthomap


def test_title():
    assert orthomap.__title__ == 'orthomap'


def test_version():
    assert orthomap.__version__ == '0.0.1'


def test_license():
    assert orthomap.__license__ == 'GPL-3'



#!/usr/bin/python
# -*- coding: UTF-8 -*-


"""
Author: Kristian K Ullrich
date: April 2023
email: ullrich@evolbio.mpg.de
License: GPL-3
"""

import os
import sys
import argparse
import pandas as pd
from orthomap import of2orthomap, qlin
from ete3 import NCBITaxa


def define_parser():
    """
    A helper function for using `eggnog2orthomap.py` via the terminal.

    :return: An argparse.ArgumentParser.

    :rtype: argparse.ArgumentParser
    """
    eggnog2orthomap_example = '''example:

    #
    $ eggnog2orthomap -qt 10090 -og
    '''
    parser = argparse.ArgumentParser(
        prog='eggnog2orthomap',
        usage='%(prog)s [options] [<arguments>...]',
        description='extract orthomap from eggnog output for query species',
        epilog=eggnog2orthomap_example,
        formatter_class=argparse.RawDescriptionHelpFormatter)
    add_argparse_args(parser=parser)
    return parser


def add_argparse_args(parser: argparse.ArgumentParser):
    """
    This function attaches individual argument specifications to the parser.

    :param parser: An argparse.ArgumentParser.

    :type parser: argparse.ArgumentParser
    """
    parser.add_argument('-qt',
                        help='query species taxID (e.g. use <orthomap qlin -h> to get taxID)')
    parser.add_argument('-og',
                        help='specify eggnog <e6.og2seqs_and_species.tsv>')
    parser.add_argument('-subset',
                        help='specify file of orthologous groups to include '
                             '<e6.og2parents_and_children.new.tsv>')
    parser.add_argument('-out',
                        help='specify output file <orthomap.tsv> (default: orthomap.tsv)',
                        default='orthomap.tsv')
    parser.add_argument('-overwrite',
                        help='specify if existing output file should be overwritten (default: True)',
                        default=True,
                        type=bool)


def get_eggnog_orthomap(qt,
                        og,
                        subset=None,
                        out=None,
                        quiet=False,
                        continuity=True,
                        overwrite=True):
    """
    This function return an orthomap for a given query species and eggnog input data.

    :param qt: Query species taxID.
    :param og: Path to eggnog <e6.og2seqs_and_species.tsv> file.
    :param subset: Path to file containing orthologous groups to include.
    :param out: Path to output file.
    :param quiet: Specify if output should be quiet.
    :param continuity: Specify if continuity score should be calculated.
    :param overwrite: Specify if output should be overwritten.
    :return: A list of results such as:
             orthomap, species_list, youngest_common_counts

    :type qt: str
    :type og: str
    :type subset: str
    :type out: str
    :type quiet: bool
    :type continuity: bool
    :type overwrite: bool
    :rtype: list

    Example
    -------
    >>>
    """
    outhandle = None
    subset_dict = None
    og_continuity_score = None
    ncbi = NCBITaxa()
    qname,\
        qtid,\
        qlineage,\
        qlineagenames_dict,\
        qlineagezip,\
        qlineagenames,\
        qlineagerev,\
        qk = qlin.get_qlin(qt=qt,
                           quiet=True)
    query_lineage_topo = qlin.get_lineage_topo(qt)
    if subset is not None:
        subset_dict = {}
        with open(subset,
                  'r') as subset_ogs:
            for subset_tmp in subset_ogs:
                sog_name = subset_tmp.strip().split('\t')[0]
                subset_dict[sog_name] = []
    ogs_dict = {}
    species_list = []
    with open(og,
              'r') as ogs:
        for og_line in ogs:
            col1_taxonomic_level,\
                col2_og_name,\
                col3_number_of_species,\
                col4_number_of_members,\
                col5_comma_separated_list_of_species,\
                col6_comma_separated_list_of_members = og_line.strip().split('\t')
            col5_comma_separated_list_of_species = col5_comma_separated_list_of_species.split(',')
            if subset is not None:
                if col2_og_name not in subset_dict:
                    continue
                else:
                    if str(qtid) in col5_comma_separated_list_of_species:
                        col6_comma_separated_list_of_members = col6_comma_separated_list_of_members.split(',')
                        q_genes = [x for x in col6_comma_separated_list_of_members if x.split('.')[0] == str(qtid)]
                        ogs_dict[col2_og_name] = [col2_og_name,
                                                  col5_comma_separated_list_of_species,
                                                  q_genes]
                        species_list += col5_comma_separated_list_of_species
            else:
                if str(qtid) in col5_comma_separated_list_of_species:
                    col6_comma_separated_list_of_members = col6_comma_separated_list_of_members.split(',')
                    q_genes = [x for x in col6_comma_separated_list_of_members if x.split('.')[0] == str(qtid)]
                    ogs_dict[col2_og_name] = [col2_og_name,
                                              col5_comma_separated_list_of_species,
                                              q_genes]
                    species_list += col5_comma_separated_list_of_species
    species_list = list(set(species_list))
    if len(species_list) == 0:
        print('\nError <-qt>: query species taxID not in eggnog results, please check taxID.')
        sys.exit()
    species_names = [qlin.get_qlin(qt=x,
                                   quiet=True)[0] for x in species_list]
    species_list_df = pd.DataFrame(species_names,
                                   columns=['species'])
    species_list_df['taxID'] = [int(x) for x in species_list]
    species_list_df['lineage'] = species_list_df.apply(lambda x: ncbi.get_lineage(x[1]),
                                                       axis=1)
    species_list_df['youngest_common'] = [qlin.get_youngest_common(qlineage,
                                                                   x) for x in species_list_df.lineage]
    species_list_df['youngest_name'] = [list(x.values())[0] for x in [ncbi.get_taxid_translator([x])
                                                                      for x in list(species_list_df.youngest_common)]]
    if not quiet:
        print(qname)
        print(qt)
        print(species_list_df)
    youngest_common_counts_df = of2orthomap.get_youngest_common_counts(qlineage,
                                                                       species_list_df)
    for node in query_lineage_topo.traverse('postorder'):
        nsplit = node.name.split('/')
        if len(nsplit) == 3:
            node.add_feature('species_count',
                             list(youngest_common_counts_df[youngest_common_counts_df.PStaxID.isin(
                                 [int(nsplit[1])])].counts)[0])
    og_dict = {}
    continuity_dict = {}
    for og in ogs_dict.keys():
        og_tmp = ogs_dict[og]
        og_hits = [int(x) for x in og_tmp[1]]
        # get list of the youngest common between query and all other species
        og_hits_youngest_common = list(species_list_df.youngest_common[
                                           [x for x, y in enumerate(species_list_df.taxID)
                                            if y in og_hits]])
        # evaluate all youngest common nodes to retain the oldest of them and assign as the orthogroup
        # ancestral state (gene age)
        if len(og_hits_youngest_common) > 0:
            og_oldest_common = qlin.get_oldest_common(qlineage,
                                                      og_hits_youngest_common)
            og_dict[og_tmp[0]] = og_oldest_common
            if continuity:
                continuity_dict[og_tmp[0]] = \
                    of2orthomap.get_youngest_common_counts(qlineage,
                                                           pd.DataFrame(og_hits_youngest_common,
                                                                        columns=['youngest_common'])).counts
    if continuity:
        youngest_common_counts_df = youngest_common_counts_df.join(pd.DataFrame.from_dict(continuity_dict))
    omap = []
    if out:
        if os.path.exists(out) and not overwrite:
            print('\nError <-overwrite>: output file exists, please set to True if it should be overwritten\n')
            sys.exit()
        outhandle = open(out, 'w')
        if continuity:
            outhandle.write('seqID\tOrthogroup\tPSnum\tPStaxID\tPSname\tPScontinuity\n')
        else:
            outhandle.write('seqID\tOrthogroup\tPSnum\tPStaxID\tPSname\n')
    for og in ogs_dict.keys():
        og_tmp = ogs_dict[og]
        og_ps = qlineagenames[qlineagenames['PStaxID'] ==
                              str(og_dict[og_tmp[0]])].values.tolist()[0]
        og_ps_join = '\t'.join(og_ps)
        if continuity:
            og_continuity_score = of2orthomap.get_continuity_score(og_tmp[0],
                                                                   youngest_common_counts_df)
            if out:
                if continuity:
                    [outhandle.write(x.replace(' ', '') + '\t' + og_tmp[0] + '\t' + og_ps_join + '\t' +
                                     str(og_continuity_score) + '\n') for x in og_tmp[2]]
                else:
                    [outhandle.write(x.replace(' ', '') + '\t' + og_tmp[0] + '\t' + og_ps_join + '\n')
                     for x in og_tmp[2]]
        if continuity:
            omap += [[x.replace(' ', ''), og_tmp[0], og_ps[0], og_ps[1], og_ps[2], og_continuity_score]
                     for x in og_tmp[2]]
        else:
            omap += [[x.replace(' ', ''), og_tmp[0], og_ps[0], og_ps[1], og_ps[2]]
                     for x in og_tmp[2]]
    if out:
        outhandle.close()
    omap_df = pd.DataFrame(omap)
    if continuity:
        omap_df.columns = ['seqID',
                           'Orthogroup',
                           'PSnum',
                           'PStaxID',
                           'PSname',
                           'PScontinuity']
    else:
        omap_df.columns = ['seqID',
                           'Orthogroup',
                           'PSnum',
                           'PStaxID',
                           'PSname']
    omap_df['PSnum'] = [int(x) for x in list(omap_df['PSnum'])]
    return [omap_df,
            species_list_df,
            youngest_common_counts_df]


def main():
    """
    The main function that is being called when `eggnog2orthomap` is used via the terminal.
    """
    parser = define_parser()
    args = parser.parse_args()
    print(args)
    if not args.qt:
        parser.print_help()
        print('\nError <-qt>: Please specify query species taxID')
        sys.exit()
    if not args.og:
        parser.print_help()
        print('\nError <-og>: Please specify eggnog <e6.og2seqs_and_species.tsv>')
        sys.exit()
    get_eggnog_orthomap(args.qt,
                        args.og,
                        subset=args.subset,
                        out=args.out,
                        overwrite=args.overwrite)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: UTF-8 -*-

import os
import argparse
from orthomap import ncbitax


def test_define_parser():
    parse = ncbitax.define_parser()
    assert isinstance(parse, argparse.ArgumentParser)


def test_update_ncbi():
    path = os.path.expanduser('~/.etetoolkit/taxa.sqlite')
    path_exist = os.path.exists(path)
    if not path_exist:
        ncbitax.update_ncbi()
        assert os.path.exists(path)



#!/usr/bin/python
# -*- coding: UTF-8 -*-


"""
Author: Kristian K Ullrich
date: April 2023
email: ullrich@evolbio.mpg.de
License: GPL-3
"""


import os
import sys
import zipfile
import argparse
import pandas as pd
from orthomap import qlin
from ete3 import NCBITaxa


def define_parser():
    """
    A helper function for using `of2orthomap.py` via the terminal.

    :return: An argparse.ArgumentParser.

    :rtype: argparse.ArgumentParser
    """
    of2orthomap_example = '''of2orthomap example:

    # download OrthoFinder example:
    $ wget https://zenodo.org/record/7796253/files/ensembl_105_orthofinder_Orthogroups.GeneCount.tsv.zip
    $ wget https://zenodo.org/record/7796253/files/ensembl_105_orthofinder_Orthogroups.tsv.zip
    $ wget https://zenodo.org/record/7796253/files/ensembl_105_orthofinder_species_list.tsv
    
    # extract orthomap:
    $ of2orthomap -seqname Danio_rerio.GRCz11.cds.longest -qt 7955 \\
      -sl ensembl_105_orthofinder_species_list.tsv \\
      -oc ensembl_105_orthofinder_Orthogroups.GeneCount.tsv.zip \\
      -og ensembl_105_orthofinder_Orthogroups.tsv.zip
    '''
    parser = argparse.ArgumentParser(
        prog='of2orthomap',
        usage='%(prog)s [options] [<arguments>...]',
        description='extract orthomap from OrthoFinder output for query species',
        epilog=of2orthomap_example,
        formatter_class=argparse.RawDescriptionHelpFormatter)
    add_argparse_args(parser=parser)
    return parser


def add_argparse_args(parser: argparse.ArgumentParser):
    """
    This function attaches individual argument specifications to the parser.

    :param parser: An argparse.ArgumentParser.

    :type parser: argparse.ArgumentParser
    """
    parser.add_argument('-seqname',
                        help='sequence name of the query species in OrthoFinder '
                             '(see column names of  <Orthogroups.tsv>)')
    parser.add_argument('-qt',
                        help='query species taxID (e.g. use <orthomap qlin -h> to get taxID)')
    parser.add_argument('-sl',
                        help='species list as <OrthoFinder name><tab><species taxID> '
                             '(only samples in this list will be processed)')
    parser.add_argument('-oc',
                        help='specify OrthoFinder <Orthogroups.GeneCounts.tsv> (see Orthogroups directory)')
    parser.add_argument('-og',
                        help='specify OrthoFinder <Orthogroups.tsv> (see Orthogroups directory)')
    parser.add_argument('-out',
                        help='specify output file <orthomap.tsv> (default: orthomap.tsv)',
                        default='orthomap.tsv')
    parser.add_argument('-overwrite',
                        help='specify if existing output file should be overwritten (default: True)',
                        default=True,
                        type=bool)


def get_orthomap(seqname,
                 qt,
                 sl,
                 oc,
                 og,
                 out=None,
                 quiet=False,
                 continuity=True,
                 overwrite=True):
    """
    This function return an orthomap for a given query species and OrthoFinder input data.

    :param seqname: Sequence name of the query species used for OrthoFinder comparison.
    :param qt: Query species taxID.
    :param sl: Path to species list file containing <OrthoFinder name><tab><species taxID>.
    :param oc: Path to OrthoFinder result <Orthogroups.GeneCounts.tsv> file.
    :param og: Path to OrthoFinder result <Orthogroups.tsv> file.
    :param out: Path to output file.
    :param quiet: Specify if output should be quiet.
    :param continuity: Specify if continuity score should be calculated.
    :param overwrite: Specify if output should be overwritten.
    :return: A list of results such as:
             orthomap, species_list, youngest_common_counts

    :type seqname: str
    :type qt: str
    :type sl: str
    :type oc: str
    :type og: str
    :type out: str
    :type quiet: bool
    :type continuity: bool
    :type overwrite: bool
    :rtype: list

    Example
    -------
    >>> from orthomap import datasets, of2orthomap, qlin
    >>> datasets.ensembl105(datapath='.')
    >>> query_orthomap = of2orthomap.get_orthomap(
    >>>     seqname='Danio_rerio.GRCz11.cds.longest',
    >>>     qt='7955',
    >>>     sl='ensembl_105_orthofinder_species_list.tsv',
    >>>     oc='ensembl_105_orthofinder_Orthogroups.GeneCount.tsv',
    >>>     og='ensembl_105_orthofinder_Orthogroups.tsv',
    >>>     out=None,
    >>>     quiet=False,
    >>>     continuity=True,
    >>>     overwrite=True)
    >>> query_orthomap
    """
    outhandle = None
    og_continuity_score = None
    ncbi = NCBITaxa()
    qname, \
        qtid, \
        qlineage, \
        qlineagenames_dict, \
        qlineagezip, \
        qlineagenames, \
        qlineagerev, \
        qk = qlin.get_qlin(qt=qt,
                           quiet=True)
    query_lineage_topo = qlin.get_lineage_topo(qt)
    species_list = pd.read_csv(sl, sep='\t', header=None)
    species_list.columns = ['species', 'taxID']
    species_list['lineage'] = species_list.apply(lambda x: ncbi.get_lineage(x[1]),
                                                 axis=1)
    species_list['youngest_common'] = [qlin.get_youngest_common(qlineage, x) for x in species_list.lineage]
    species_list['youngest_name'] = [list(x.values())[0] for x in [ncbi.get_taxid_translator([x])
                                                                   for x in list(species_list.youngest_common)]]
    if not quiet:
        print(seqname)
        print(qname)
        print(qt)
        print(species_list)
    youngest_common_counts_df = get_youngest_common_counts(qlineage,
                                                           species_list)
    for node in query_lineage_topo.traverse('postorder'):
        nsplit = node.name.split('/')
        if len(nsplit) == 3:
            node.add_feature('species_count',
                             list(youngest_common_counts_df[youngest_common_counts_df.PStaxID.isin(
                                 [int(nsplit[1])])].counts)[0])
    oc_og_dict = {}
    continuity_dict = {}
    if os.path.basename(oc).split('.')[-1] == 'zip':
        oc_zip = zipfile.Path(oc, at='.'.join(os.path.basename(oc).split('.')[:-1]))
        oc_lines = oc_zip.open()
    else:
        oc_lines = open(oc,
                        'r')
    oc_species = next(oc_lines)
    if type(oc_species) == bytes:
        oc_species = oc_species.decode('utf-8').strip().split('\t')
    else:
        oc_species = oc_species.strip().split('\t')
    oc_qidx = [x for x, y in enumerate(oc_species) if y == seqname]
    if len(oc_qidx) == 0:
        print('\nError <-qname>: query species name not in orthofinder results, please check spelling\n'
              'e.g. <head -1 Orthogroups.GeneCounts.tsv>')
        sys.exit()
    for oc_line in oc_lines:
        if type(oc_line) == bytes:
            oc_og = oc_line.decode('utf-8').strip().split('\t')
        else:
            oc_og = oc_line.strip().split('\t')
        if int(oc_og[oc_qidx[0]]) == 0:
            continue
        if int(oc_og[oc_qidx[0]]) > 0:
            oc_og_hits = [oc_species[x+1] for x, y in enumerate(oc_og[1::][::-1][1::][::-1]) if int(y) > 0]
            # get list of the youngest common between query and all other species
            oc_og_hits_youngest_common = list(species_list.youngest_common[
                                                  [x for x, y in enumerate(species_list.species)
                                                   if y in oc_og_hits]])
            # evaluate all youngest common nodes to retain the oldest of them and assign as the orthogroup
            # ancestral state (gene age)
            if len(oc_og_hits_youngest_common) > 0:
                oc_og_oldest_common = qlin.get_oldest_common(qlineage,
                                                             oc_og_hits_youngest_common)
                oc_og_dict[oc_og[0]] = oc_og_oldest_common
                if continuity:
                    continuity_dict[oc_og[0]] = get_youngest_common_counts(
                        qlineage,
                        pd.DataFrame(oc_og_hits_youngest_common,
                                     columns=['youngest_common'])).counts
    oc_lines.close()
    if continuity:
        youngest_common_counts_df = youngest_common_counts_df.join(pd.DataFrame.from_dict(continuity_dict))
    omap = []
    if out:
        if os.path.exists(out) and not overwrite:
            print('\nError <-overwrite>: output file exists, please set to True if it should be overwritten\n')
            sys.exit()
        outhandle = open(out,
                         'w')
        if continuity:
            outhandle.write('seqID\tOrthogroup\tPSnum\tPStaxID\tPSname\tPScontinuity\n')
        else:
            outhandle.write('seqID\tOrthogroup\tPSnum\tPStaxID\tPSname\n')
    if os.path.basename(og).split('.')[-1] == 'zip':
        og_zip = zipfile.Path(og,
                              at='.'.join(os.path.basename(og).split('.')[:-1]))
        og_lines = og_zip.open()
    else:
        og_lines = open(og,
                        'r')
    og_species = next(og_lines)
    if type(og_species) == bytes:
        og_species = og_species.decode('utf-8').strip().split('\t')
    else:
        og_species = og_species.strip().split('\t')
    og_qidx = [x for x, y in enumerate(og_species) if y == seqname]
    if len(oc_qidx) == 0:
        print('\nError <-qname>: query species name not in orthofinder results, please check spelling\n'
              'e.g. <head -1 Orthogroups.tsv>')
        sys.exit()
    for og_line in og_lines:
        if type(og_line) == bytes:
            og_og = og_line.decode('utf-8').strip().split('\t')
        else:
            og_og = og_line.strip().split('\t')
        if og_og[0] not in oc_og_dict:
            continue
        else:
            og_ps = qlineagenames[qlineagenames['PStaxID'] ==
                                  str(oc_og_dict[og_og[0]])].values.tolist()[0]
            og_ps_join = '\t'.join(og_ps)
            if continuity:
                og_continuity_score = get_continuity_score(og_name=og_og[0],
                                                           youngest_common_counts_df=youngest_common_counts_df)
            if out:
                if continuity:
                    [outhandle.write(x.replace(' ', '') + '\t' + og_og[0] + '\t' + og_ps_join + '\t' +
                                     str(og_continuity_score) + '\n') for x in og_og[og_qidx[0]].split(',')]
                else:
                    [outhandle.write(x.replace(' ', '') + '\t' + og_og[0] + '\t' + og_ps_join + '\n')
                     for x in og_og[og_qidx[0]].split(',')]
        if continuity:
            omap += [[x.replace(' ', ''), og_og[0], og_ps[0], og_ps[1], og_ps[2], og_continuity_score]
                     for x in og_og[og_qidx[0]].split(',')]
        else:
            omap += [[x.replace(' ', ''), og_og[0], og_ps[0], og_ps[1], og_ps[2]]
                     for x in og_og[og_qidx[0]].split(',')]
    og_lines.close()
    if out:
        outhandle.close()
    omap_df = pd.DataFrame(omap)
    if continuity:
        omap_df.columns = ['seqID',
                           'Orthogroup',
                           'PSnum',
                           'PStaxID',
                           'PSname',
                           'PScontinuity']
    else:
        omap_df.columns = ['seqID',
                           'Orthogroup',
                           'PSnum',
                           'PStaxID',
                           'PSname']
    omap_df['PSnum'] = [int(x) for x in list(omap_df['PSnum'])]
    return [omap_df,
            species_list,
            youngest_common_counts_df]


def get_counts_per_ps(omap_df,
                      psnum_col='PSnum',
                      pstaxid_col='PStaxID',
                      psname_col='PSname'):
    """
    This function return counts per phylostratum.

    :param omap_df: DataFrame with orthomap results.
    :param psnum_col: Specify PSnum column name.
    :param pstaxid_col: Specify PStaxID column name.
    :param psname_col: Specify PSname column name.
    :return: DataFrame with counts per phylostratum.

    :type omap_df: pandas.DataFrame
    :type psnum_col: str
    :type pstaxid_col: str
    :type psname_col: str
    :rtype: pandas.DataFrame

    Example
    -------
    >>> from orthomap import datasets, of2orthomap, qlin
    >>> datasets.ensembl105(datapath='.')
    >>> query_orthomap = of2orthomap.get_orthomap(
    >>>     seqname='Danio_rerio.GRCz11.cds.longest',
    >>>     qt='7955',
    >>>     sl='ensembl_105_orthofinder_species_list.tsv',
    >>>     oc='ensembl_105_orthofinder_Orthogroups.GeneCount.tsv',
    >>>     og='ensembl_105_orthofinder_Orthogroups.tsv',
    >>>     out=None,
    >>>     quiet=False,
    >>>     continuity=True,
    >>>     overwrite=True)
    >>> of2orthomap.get_counts_per_ps(
    >>>     omap_df=query_orthomap[0],
    >>>     psnum_col='PSnum',
    >>>     pstaxid_col='PStaxID',
    >>>     psname_col='PSname')
    """
    counts_df = pd.DataFrame(omap_df.value_counts(psnum_col))
    counts_df.columns = ['counts']
    counts_df.reset_index(inplace=True)
    if pstaxid_col:
        counts_df = counts_df.merge(omap_df[~omap_df.duplicated(psnum_col)][[psnum_col,
                                                                             pstaxid_col]],
                                    on=psnum_col)
    if psname_col:
        counts_df = counts_df.merge(omap_df[~omap_df.duplicated(psnum_col)][[psnum_col,
                                                                             psname_col]],
                                    on=psnum_col)
    counts_df.set_index(psnum_col,
                        inplace=True,
                        drop=False)
    counts_df = counts_df.sort_index()
    return counts_df


def get_youngest_common_counts(qlineage,
                               species_list):
    """
    This function return LCA counts for a given query species lineage.

    :param qlineage: Query lineage information.
    :param species_list: Species list.
    :return: DataFrame with LCA counts.

    :type qlineage: list
    :type species_list: pandas.DataFrame
    :rtype: pandas.DataFrame

    Example
    -------
    >>>
    """
    counts_df = pd.DataFrame(qlineage,
                             columns=['lineage'])
    counts_df.set_index('lineage',
                        inplace=True)
    counts_df = pd.concat([counts_df,
                           species_list['youngest_common'].value_counts()],
                          join='outer',
                          axis=1)
    counts_df.columns = ['counts']
    counts_df['PStaxID'] = counts_df.index.values
    counts_df['PSnum'] = list(range(len(counts_df['PStaxID'])))
    return counts_df


def get_continuity_score(og_name,
                         youngest_common_counts_df):
    """
    This function calculates a continuity score for a given orthologous group
    and its corresponding LCA counts.

    :param og_name: Orthologous group name.
    :param youngest_common_counts_df: DataFrame with LCA counts.
    :return: Continuity score.

    :type og_name: str
    :type youngest_common_counts_df: pandas.DataFrame
    :rtype: float

    Example
    -------
    >>>
    """
    og_continuity_score = 0.0
    og_df = youngest_common_counts_df[~youngest_common_counts_df['counts'].isna()][og_name]
    og_lca = (~og_df.isna()).idxmax()
    og_lca_pos = og_df.index.get_loc(og_lca)
    og_lca_df = og_df.iloc[og_lca_pos:]
    og_lca_df_counts = og_lca_df.isna().value_counts()
    if False in og_lca_df_counts:
        og_continuity_score = og_lca_df_counts[False] / len(og_lca_df)
    return og_continuity_score


def main():
    """
    The main function that is being called when `of2orthomap` is used via the terminal.
    """
    parser = define_parser()
    args = parser.parse_args()
    print(args)
    if not args.seqname:
        parser.print_help()
        print('\nError <-seqname>: Please specify query species name in OrthoFinder and taxID')
        sys.exit()
    if not args.qt:
        parser.print_help()
        print('\nError <-qt>: Please specify query species taxID')
        sys.exit()
    if not args.sl:
        parser.print_help()
        print('\nError <-sl>: Please specify species list as <OrthoFinder name><tab><species taxID>')
        sys.exit()
    if not args.oc:
        parser.print_help()
        print('\nError <-oc>: Please specify OrthoFinder <Orthogroups.GeneCounts.tsv> (see Orthogroups directory)')
        sys.exit()
    if not args.og:
        parser.print_help()
        print('\nError <-og>: Please specify OrthoFinder <Orthogroups.tsv> (see Orthogroups directory)')
        sys.exit()
    get_orthomap(seqname=args.seqname,
                 qt=args.qt,
                 sl=args.sl,
                 oc=args.oc,
                 og=args.og,
                 out=args.out,
                 quiet=False,
                 continuity=True,
                 overwrite=args.overwrite)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: UTF-8 -*-


"""
Author: Kristian K Ullrich
date: April 2023
email: ullrich@evolbio.mpg.de
License: GPL-3
"""


import os
import sys
import argparse
import gzip
import pandas as pd


def define_parser():
    """
    A helper function for using `gtf2t2g.py` via the terminal.

    :return: An argparse.ArgumentParser.

    :rtype: argparse.ArgumentParser
    """
    gtf2t2g_example = '''gtf2t2g example:
    
    # to get GTF from Mus musculus on Linux run:
    $ wget https://ftp.ensembl.org/pub/release-108/gtf/mus_musculus/Mus_musculus.GRCm39.108.chr.gtf.gz

    # on Mac:
    $ curl https://ftp.ensembl.org/pub/release-108/gtf/mus_musculus/Mus_musculus.GRCm39.108.chr.gtf.gz --remote-name

    # create t2g from GTF:
    $ gtf2t2g -i Mus_musculus.GRCm39.108.chr.gtf.gz -o Mus_musculus.GRCm39.108.chr.gtf.t2g.tsv -g -b -p -v -s
    '''
    parser = argparse.ArgumentParser(
        prog='gtf2t2g',
        usage='%(prog)s [options] [<arguments>...]',
        description='extract transcript to gene table from GTF',
        epilog=gtf2t2g_example,
        formatter_class=argparse.RawDescriptionHelpFormatter)
    add_argparse_args(parser=parser)
    return parser


def add_argparse_args(parser: argparse.ArgumentParser):
    """
    This function attaches individual argument specifications to the parser.

    :param parser: An argparse.ArgumentParser.

    :type parser: argparse.ArgumentParser
    """
    parser.add_argument('-i',
                        help='specify GTF input file')
    parser.add_argument('-o',
                        help='specify output file [optional]')
    parser.add_argument('-g',
                        help='specify if gene names should be appended if they exist',
                        action='store_true')
    parser.add_argument('-b',
                        help='specify if gene biotype should be appended if they exist',
                        action='store_true')
    parser.add_argument('-p',
                        help='specify if protein id should be appended if they exist',
                        action='store_true')
    parser.add_argument('-v',
                        help='specify if gene/transcript/protein version should be appended',
                        action='store_true')
    parser.add_argument('-s',
                        help='specify if summary should be printed',
                        action='store_true')
    parser.add_argument('-q',
                        help='specify if output should be quite',
                        action='store_true')
    parser.add_argument('-overwrite',
                        help='specify if existing output file should be overwritten (default: True)',
                        default=True,
                        type=bool)


def _information_based_on_key(infosplit,
                              key,
                              q,
                              lines,
                              version=False):
    """
    A helper function to split the information column of GTF file.

    :param infosplit: Split information column of GTF file.
    :param key: Keyword.
    :param q: Specify if output should be quiet.
    :param lines: Line of GTF file.
    :param version: Specify if gene/transcript/protein version should be appended.
    :return: A list of parsed information from a GTF info field such as:
             infos, output

    :type infosplit: list
    :type key: str
    :type q: bool
    :type lines: str
    :type version: bool
    :rtype: list

    Example
    -------
    >>>
    """
    output = None
    if version:
        infos = [x for x in infosplit if key in x and f'havana_{key}' not in x]
    else:
        infos = [x for x in infosplit if key in x]
    if infos:
        if len(infos) == 1:
            output = infos[0]
            output = output.replace(key, '').replace(' ', '').replace('"', '')
        else:
            if not q:
                print(f'duplicated {key} field:\t' + lines)
    else:
        if not q:
            print(f'no {key} field:\t' + lines)
    return [infos, output]


def parse_gtf(gtf,
              g=False,
              b=False,
              p=False,
              v=False,
              s=False,
              output=None,
              q=False):
    """
    This function parses a GTF file to extract transcript and gene IDs.

    :param gtf: Path to GTF file.
    :param g: Specify if gene names should be appended if they exist.
    :param b: Specify if gene biotype should be appended if they exist.
    :param p: Specify if protein id should be appended if they exist.
    :param v: Specify if gene/transcript/protein version should be appended.
    :param s: Specify if summary should be printed.
    :param output: File object.
    :param q: Specify if output should be quiet.
    :return: DataFrame with transcript and gene IDs.

    :type gtf: str
    :type g: bool
    :type b: bool
    :type p: bool
    :type v: bool
    :type s: bool
    :type output: file object
    :type q: bool
    :rtype: pandas.DataFrame

    Example
    -------
    >>> from orthomap import datasets, gtf2t2g
    >>> # get gene to transcript table for Danio rerio
    >>> # https://ftp.ensembl.org/pub/release-105/gtf/danio_rerio/Danio_rerio.GRCz11.105.gtf.gz
    >>> gtf_file = datasets.zebrafish_gtf(datapath='.')
    >>> query_species_t2g = gtf2t2g.parse_gtf(gtf=gtf_file,\
    >>>     g=True, b=True, p=True, v=True, s=True, q=True)
    >>> query_species_t2g
    """
    if gtf.endswith('gz'):
        gtf_handle = gzip.open(gtf,
                               'rt')
    else:
        gtf_handle = open(gtf,
                          'rt')
    t2g = {}
    t2p = {}
    tc = 0
    gc = 0
    pc = 0
    dc = 0
    for lines in gtf_handle:
        if len(lines) == 0 or lines[0] == '#':
            continue
        line = lines.strip().split('\t')
        if line[2] == 'transcript':
            gname_first = None
            gtype_first = None
            gid_first_version = None
            tid_first_version = None
            infosplit = line[8].strip().split(';')
            gid, gid_first = _information_based_on_key(infosplit=infosplit,
                                                       key='gene_id',
                                                       q=q,
                                                       lines=lines)
            if not gid:
                continue
            tid, tid_first = _information_based_on_key(infosplit=infosplit,
                                                       key='transcript_id',
                                                       q=q,
                                                       lines=lines)
            if not tid:
                continue
            if g:
                _, gname_first = _information_based_on_key(infosplit=infosplit,
                                                           key='gene_name',
                                                           q=q,
                                                           lines=lines)
            if b:
                _, gtype_first = _information_based_on_key(infosplit=infosplit,
                                                           key='gene_biotype',
                                                           q=q,
                                                           lines=lines)
            if v:
                _, gv_first = _information_based_on_key(infosplit=infosplit,
                                                        key='gene_version',
                                                        q=q,
                                                        lines=line,
                                                        version=True)
                _, tv_first = _information_based_on_key(infosplit=infosplit,
                                                        key='transcript_version',
                                                        q=q,
                                                        lines=line,
                                                        version=True)
                if gv_first:
                    gid_first_version = gid_first + '.' + gv_first
                if tv_first:
                    tid_first_version = tid_first + '.' + tv_first
            if gid_first in t2g:
                if tid_first in t2g[gid_first]:
                    dc += 1
                    if not q:
                        print('duplicated gid-tid: ' + gid_first + ' ' + tid_first)
                    continue
                if tid_first not in t2g[gid_first]:
                    tc += 1
                    t2g[gid_first][tid_first] = [
                        gid_first,
                        gid_first_version,
                        tid_first,
                        tid_first_version,
                        gname_first,
                        gtype_first, ]
            if gid_first not in t2g:
                gc += 1
                tc += 1
                t2g[gid_first] = {}
                t2g[gid_first][tid_first] = [
                    gid_first,
                    gid_first_version,
                    tid_first,
                    tid_first_version,
                    gname_first,
                    gtype_first, ]
        if line[2] == 'CDS':
            if p:
                pid_first = None
                pid_first_version = None
                tid_first = None
                pv_first = None
                infosplit = line[8].strip().split(';')
                tid, tid_first = _information_based_on_key(infosplit=infosplit,
                                                           key='transcript_id',
                                                           q=q,
                                                           lines=lines)
                tid = [x for x in infosplit if 'transcript_id' in x]
                if not tid:
                    continue
                pid, pid_first = _information_based_on_key(infosplit=infosplit,
                                                           key='protein_id',
                                                           q=q,
                                                           lines=lines)
                if not pid:
                    continue
                if v:
                    _, pv_first = _information_based_on_key(infosplit=infosplit,
                                                            key='protein_version',
                                                            q=q,
                                                            lines=lines)
                    if pv_first:
                        pid_first_version = pid_first + '.' + pv_first
                if tid_first in t2p:
                    continue
                if tid_first not in t2p:
                    pc += 1
                    t2p[tid_first] = [pid_first, pid_first_version]
    for gidk in sorted(t2g.keys()):
        for tidk in sorted(t2g[gidk].keys()):
            pidk = None
            pidk_v = None
            if tidk in t2p:
                pidk = t2p[tidk][0]
                pidk_v = t2p[tidk][1]
            t2g[gidk][tidk] = t2g[gidk][tidk] + [pidk, pidk_v]
            if output:
                output.write('\t'.join([str(x) for x in t2g[gidk][tidk]]) + '\n')
    if s:
        summary_text = f"""{str(gc)} gene_id found
{str(tc)} transcript_id found
{str(tc)} protein_id found
{str(dc)} duplicated"""
        print(summary_text)
    t2g_df = pd.DataFrame.from_dict(
        {(i, j): t2g[i][j] for i in t2g.keys() for j in t2g[i].keys()},
        orient='index',
        columns=[
            'gene_id',
            'gene_id_version',
            'transcript_id',
            'transcript_id_version',
            'gene_name',
            'gene_type',
            'protein_id',
            'protein_id_version', ], )
    t2g_df.sort_values(by='gene_id',
                       inplace=True)
    t2g_df.reset_index(drop=True,
                       inplace=True)
    gtf_handle.close()
    return t2g_df


def main():
    """
    The main function that is being called when `gtf2t2g` is used via the terminal.
    """
    parser = define_parser()
    args = parser.parse_args()
    print(args)
    if not args.i:
        parser.print_help()
        print('\nError <-i>: Please specify GTF input file')
        sys.exit()
    if args.o:
        if os.path.exists(args.o) and not args.overwrite:
            print('\nError <-overwrite>: output file exists, please set to True if it should be overwritten\n')
            sys.exit()
        output = open(args.o, 'w')
    else:
        output = sys.stdout
    parse_gtf(gtf=args.i,
              g=args.g,
              b=args.b,
              p=args.p,
              v=args.v,
              s=args.s,
              output=output,
              q=args.q)
    output.close()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: UTF-8 -*-

import argparse
import pandas as pd
from orthomap import cds2aa


def test_define_parser():
    parse = cds2aa.define_parser()
    assert isinstance(parse, argparse.ArgumentParser)


#!/usr/bin/python
# -*- coding: UTF-8 -*-

import argparse
import pandas as pd
from orthomap import datasets, gtf2t2g


file = datasets.mouse_gtf(datapath='/tmp')
expected_columns = [
    'gene_id',
    'gene_id_version',
    'transcript_id',
    'transcript_id_version',
    'gene_name',
    'gene_type',
    'protein_id',
    'protein_id_version']


def test_define_parser():
    parse = gtf2t2g.define_parser()
    assert isinstance(parse, argparse.ArgumentParser)


def test_gtf2t2g_only_filename():
    output = gtf2t2g.parse_gtf(file, q=True)
    assert (output.columns == expected_columns).all()
    assert isinstance(output, pd.DataFrame)
    empty_columns = [
        'gene_id_version',
        'transcript_id_version',
        'gene_name',
        'gene_type',
        'protein_id',
        'protein_id_version']
    for col in empty_columns:
        assert len(output[col].unique()) == 1
        assert output[col].unique()[0] == 'None' or output[col].unique()[0] is None


def test_gtf2t2g_with_gene_type_and_versions():
    output = gtf2t2g.parse_gtf(file, g=True, b=True, v=True, q=True)
    assert (output.columns == expected_columns).all()
    assert isinstance(output, pd.DataFrame)
    empty_columns = ['protein_id', 'protein_id_version']
    for col in empty_columns:
        assert len(output[col].unique()) == 1
        assert output[col].unique()[0] is None


def test_gtf2t2g_with_gene_protein_id_version():
    output = gtf2t2g.parse_gtf(file, g=True, p=True, q=True)
    assert (output.columns == expected_columns).all()
    assert isinstance(output, pd.DataFrame)
    empty_columns = [
        'gene_id_version',
        'transcript_id_version',
        'gene_type',
        'protein_id_version']
    for col in empty_columns:
        assert len(output[col].unique()) == 1
        assert output[col].unique()[0] == 'None' or output[col].unique()[0] is None


def test_mus_musculus():
    expected_first_row = [
        'ENSMUSG00000000001',
        'ENSMUSG00000000001.5',
        'ENSMUST00000000001',
        'ENSMUST00000000001.5',
        'Gnai3',
        None,
        'ENSMUSP00000000001',
        'ENSMUSP00000000001.5']
    expected_last_row = [
        'ENSMUSG00002076992',
        'ENSMUSG00002076992.1',
        'ENSMUST00020181762',
        'ENSMUST00020181762.1',
        '7SK',
        None,
        None,
        None]
    output = gtf2t2g.parse_gtf(file, g=True, p=True, s=True, q=True, v=True)
    assert (output.iloc[0].values == expected_first_row).all()
    assert (output.iloc[-1].values == expected_last_row).all()


def test_duplicates():
    # Need KU help here.
    pass



#!/usr/bin/python
# -*- coding: UTF-8 -*-


"""
Author: Kristian K Ullrich
date: April 2023
email: ullrich@evolbio.mpg.de
License: GPL-3
"""


import sys
import argparse
import pandas as pd
from ete3 import NCBITaxa, Tree


def define_parser():
    """
    A helper function for using `qlin.py` via the terminal.

    :return: An argparse.ArgumentParser.

    :rtype: argparse.ArgumentParser
    """
    qlin_example = '''qlin example:

    # get query lineage to be used with orthomap later on using query species taxID
    # Mus musculus; 10090
    $ qlin -qt 10090

    # using query species name
    $ qlin -q "Mus musculus"
    '''
    parser = argparse.ArgumentParser(
        prog='qlin',
        usage='%(prog)s [options] [<arguments>...]',
        description='get query lineage based on ncbi taxonomy',
        epilog=qlin_example,
        formatter_class=argparse.RawDescriptionHelpFormatter)
    add_argparse_args(parser=parser)
    return parser


def add_argparse_args(parser: argparse.ArgumentParser):
    """
    This function attaches individual argument specifications to the parser.

    :param parser: An argparse.ArgumentParser.

    :type parser: argparse.ArgumentParser
    """
    parser.add_argument('-q',
                        help='query species name')
    parser.add_argument('-qt',
                        help='query species taxID')


def get_qlin(ncbi=None,
             q=None,
             qt=None,
             quiet=False):
    """
    This function searches the NCBI taxonomic database for results matching the
    query name or query taxID.

    Note that if the user specifies both the name and the taxID of a species,
    the returning result is based on the taxID.

    :param ncbi: The NCBI taxonomic database.
    :param q: The name of the queried species.
    :param qt: The taxID of the queried species.
    :param quiet: Specify if output should be quiet.
    :return: A list of information for the queried species such as:
             query name, query taxID, query lineage, query lineage dictionary, query lineage zip,
             query lineage names, reverse query lineage, query kingdom

    :type ncbi: ete3.NCBITaxa
    :type q: str
    :type qt: str
    :type quiet: bool
    :rtype: list

    Example
    -------
    >>> from ete3 import NCBITaxa
    >>> from orthomap import qlin
    >>> qlin.get_qlin(q='Danio rerio')
    """
    qtid = None
    qname = None
    qk = None
    if ncbi is None:
        ncbi = NCBITaxa()
    if qt:
        taxid2name = ncbi.get_taxid_translator([int(qt)])
        qtid, \
            qname = list(taxid2name.items())[0]
    if q and not qt:
        name2taxid = ncbi.get_name_translator([q])
        qname, \
            qtid = list(name2taxid.items())[0]
        qtid = qtid[0]
    qlineage = ncbi.get_lineage(qtid)
    qlineagenames_dict = ncbi.get_taxid_translator(qlineage)
    qlineagezip = [(a, qlineagenames_dict[a]) for a in qlineage]
    qlineagenames = pd.DataFrame([(x, y, qlineagenames_dict[y]) for x, y in enumerate(qlineage)],
                                 columns=['PSnum',
                                          'PStaxID',
                                          'PSname'])
    qlineagenames['PSnum'] = [str(x) for x in list(qlineagenames['PSnum'])]
    qlineagenames['PStaxID'] = [str(x) for x in list(qlineagenames['PStaxID'])]
    qlineagenames['PSname'] = [str(x) for x in list(qlineagenames['PSname'])]
    qlineagerev = qlineage[::-1]
    if qlineage[2] == 2:
        qk = 'Bacteria'
    if qlineage[2] == 2157:
        qk = 'Archea'
    if qlineage[2] == 2759:
        qk = 'Eukaryota'
    if not quiet:
        print('query name: %s' % qname)
        print('query taxID: %s' % str(qtid))
        print('query kingdom: %s' % qk)
        print(
            'query lineage names: \n%s' % str([qlineagenames_dict[x] + '(' + str(x) + ')' for x in qlineage])
        )
        print('query lineage: \n%s' % str(qlineage))
    return [qname,
            qtid,
            qlineage,
            qlineagenames_dict,
            qlineagezip,
            qlineagenames,
            qlineagerev,
            qk]


def get_lineage_topo(qt):
    """
    This function returns a species lineage as a tree object for a query species given as taxID.

    :param qt: The taxID of the queried species.
    :return: The lineage of the queried species as an ete3.Tree.

    :type qt: str
    :rtype: ete3.Tree

    Example
    -------
    >>> from orthomap import qlin
    >>> lineage_tree = qlin.get_lineage_topo(qt='10090')
    >>> lineage_tree
    """
    _, _, _, _, _, qlineagenames, _, _ = get_qlin(qt=qt,
                                                  quiet=True)
    qln = list(qlineagenames[['PSnum',
                              'PStaxID',
                              'PSname']].apply(lambda x: '/'.join(x), axis=1))
    qln = [x.replace('(', '_').replace(')', '_').replace(':', '_') for x in qln]
    tree = Tree('(' * len(qln) + ''.join([str(x) + '),' for x in qln[1::][::-1]])+str(qln[0])+');')
    return tree


def get_youngest_common(ql,
                        tl):
    """
    This function returns the lowest common ancestor (LCA) by comparing the lineage information
    of a query and a target species.

    :param ql: Query species lineage information.
    :param tl: Target species lineage information.
    :return: lowest common ancestor (LCA).

    :type ql: list
    :type tl: list
    :rtype: str

    Example
    -------
    >>> from orthomap import qlin
    >>> # get query species taxonomic lineage information
    >>> _, _, query_lineage, _, _, _, _, _ = qlin.get_qlin(q='Caenorhabditis elegans')
    >>> # get target species taxonomic lineage information
    >>> _, _, target_lineage, _, _, _, _, _ = qlin.get_qlin(q='Mus musculus')
    >>> # get youngest common node
    >>> qlin.get_youngest_common(ql=query_lineage, tl=target_lineage)
    """
    return [x for x in tl if x in ql][-1]


def get_oldest_common(ql,
                      tl):
    """
    This function returns the oldest common ancestor (OCA) by comparing the lineage information
    of a query species and a target species.

    The target species can also be a list of LCA values to find the oldest among the given LCA.

    :param ql: Query species lineage information.
    :param tl: Target species lineage information.
    :return: oldest common ancestor (OCA).

    :type ql: list
    :type tl: list
    :rtype: str

    Example
    -------
    >>> from orthomap import qlin
    >>> # get query species taxonomic lineage information
    >>> _, _, query_lineage, _, _, _, _, _ = qlin.get_qlin(q='Caenorhabditis elegans')
    >>> # get target species taxonomic lineage information
    >>> _, _, target_lineage, _, _, _, _, _ = qlin.get_qlin(q='Mus musculus')
    >>> # get oldest common node
    >>> qlin.get_oldest_common(ql=query_lineage, tl=target_lineage)
    """
    return ql[min([x for x, y in enumerate(ql) if y in tl])]


def main():
    """
    The main function that is being called when `qlin` is used via the terminal.
    """
    parser = define_parser()
    args = parser.parse_args()
    print(args)
    if not args.q and not args.qt:
        parser.print_help()
        print('\nError <-q> <-qt>: Please specify query species name or taxID')
        sys.exit()
    if args.q and args.qt:
        parser.print_help()
        print('\nWarning: Since both query species name and taxID are given taxID is used')
        sys.exit()
    get_qlin(q=args.q,
             qt=args.qt,
             quiet=False)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: UTF-8 -*-


"""
Author: Kristian K Ullrich
date: April 2023
email: ullrich@evolbio.mpg.de
License: GPL-3
"""


import sys
import argparse
from ete3 import NCBITaxa


def define_parser():
    """
    A helper function for using `ncbitax.py` via the terminal.

    :return: An argparse.ArgumentParser.

    :rtype: argparse.ArgumentParser
    """
    ncbitax_example = '''ncbitax example:

    #update ncbi taxonomy database:
    ncbitax -u
    '''
    parser = argparse.ArgumentParser(
        prog='ncbitax',
        usage='%(prog)s [options] [<arguments>...]',
        description='update local ncbi taxonomy database', epilog=ncbitax_example,
        formatter_class=argparse.RawDescriptionHelpFormatter)
    add_argparse_args(parser=parser)
    return parser


def add_argparse_args(parser: argparse.ArgumentParser):
    """
    This function attaches individual argument specifications to the parser.

    :param parser: An argparse.ArgumentParser.

    :type parser: argparse.ArgumentParser
    """
    parser.add_argument('-u',
                        help='update',
                        action='store_true')


def update_ncbi():
    """
    This function updates or downloads the NCBI taxonomy database using
    the package `ete3`. A parsed version of it will be stored at the home
    directory: `~/.etetoolkit/taxa.sqlite`.

    Example
    -------
    >>> from orthomap import ncbitax
    >>> ncbitax.update_ncbi()
    """
    ncbi = NCBITaxa()
    ncbi.update_taxonomy_database()


def main():
    """
    The main function that is being called when `ncbitax.py` is used via the terminal.
    """
    parser = define_parser()
    args = parser.parse_args()
    print(args)
    if not args.u:
        parser.print_help()
        print('\nError <-u>: Please specify if you like to update <-u>')
        sys.exit()
    if args.u:
        update_ncbi()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: UTF-8 -*-

import argparse
import pandas as pd
from ete3 import Tree
from orthomap import qlin


def test_define_parser():
    parse = qlin.define_parser()
    assert isinstance(parse, argparse.ArgumentParser)


def test_get_qlin_q_argument():
    q = 'Danio rerio'
    info = qlin.get_qlin(q=q)
    assert isinstance(info, list)
    assert info[0] == q
    assert isinstance(info[3], dict)
    assert isinstance(info[-1], str)
    for info_index in [2, 4]:
        assert isinstance(info[info_index], list)
    assert isinstance(info[5], pd.DataFrame)


def test_get_qlin_qt_argument():
    qt = '7955'
    info = qlin.get_qlin(qt=qt)
    assert isinstance(info, list)
    assert info[1] == int(qt)
    assert isinstance(info[3], dict)
    assert isinstance(info[-1], str)
    for info_index in [2, 4]:
        assert isinstance(info[info_index], list)
    assert isinstance(info[5], pd.DataFrame)


def test_get_qlin_q_and_qt_argument():
    q = 'Danio rerio'
    qt = '7955'
    info = qlin.get_qlin(q=q, qt=qt)
    assert isinstance(info, list)
    assert info[0] == q
    assert info[1] == int(qt)


def test_get_qlin_q_with_wrong_qt_argument():
    """If the name and taxid do not much then `orthomap` returns information
    based on the taxid."""
    q = 'Danio rerio'
    qt = '7956'
    info = qlin.get_qlin(q=q, qt=qt)
    assert isinstance(info, list)
    assert info[0] != q
    assert info[1] == int(qt)


def test_lineage_topo():
    qt = '7955'
    tree = qlin.get_lineage_topo(qt)
    assert isinstance(tree, Tree)


def test_get_youngest_common():
    ql = ['A', 'B', 'C']
    tl = ['Q', 'N', 'A', 'C', 'B']
    assert qlin.get_youngest_common(ql, tl) == 'B'


def test_oldest_common():
    ql = ['A', 'B', 'C']
    tl = ['Q', 'N', 'A', 'C', 'B']
    assert qlin.get_oldest_common(ql, tl) == 'A'



#!/usr/bin/python
# -*- coding: UTF-8 -*-


"""
Author: Kristian K Ullrich
date: April 2023
email: ullrich@evolbio.mpg.de
License: GPL-3
"""


import os
import sys
import argparse
from orthomap import cds2aa, gtf2t2g, ncbitax, of2orthomap, plaza2orthomap, qlin


def define_parser():
    """
    A helper function for using `orthomap` via the terminal.

    :return: An argparse.ArgumentParser.

    :rtype: argparse.ArgumentParser
    """
    parser = argparse.ArgumentParser(prog='orthomap',
                                     usage='%(prog)s <sub-command>',
                                     description='orthomap')
    subparsers = parser.add_subparsers(dest='subcommand',
                                       title='sub-commands',
                                       help='sub-commands help')
    cds2aa_example = '''cds2aa example:

    # to get CDS from Danio rerio on Linux run:
    $ wget https://ftp.ensembl.org/pub/release-105/fasta/danio_rerio/cds/Danio_rerio.GRCz11.cds.all.fa.gz
    $ gunzip Danio_rerio.GRCz11.cds.all.fa.gz

    # on Mac:
    $ curl https://ftp.ensembl.org/pub/release-105/fasta/danio_rerio/cds/Danio_rerio.GRCz11.cds.all.fa.gz --remote-name
    $ gunzip Danio_rerio.GRCz11.cds.all.fa.gz
    
    # translate and retain longest isoform from CDS fasta file:
    $ cds2aa -i Danio_rerio.GRCz11.cds.all.fa -r ENSEMBL -o Danio_rerio.GRCz11.aa.all.longest.fa
    '''
    gtf2t2g_example = '''gtf2t2g example:

    # to get GTF from Mus musculus on Linux run:
    $ wget https://ftp.ensembl.org/pub/release-108/gtf/mus_musculus/Mus_musculus.GRCm39.108.chr.gtf.gz

    # on Mac:
    $ curl https://ftp.ensembl.org/pub/release-108/gtf/mus_musculus/Mus_musculus.GRCm39.108.chr.gtf.gz --remote-name

    # create t2g from GTF:
    $ gtf2t2g -i Mus_musculus.GRCm39.108.chr.gtf.gz -o Mus_musculus.GRCm39.108.chr.gtf.t2g.tsv -g -b -p -v -s
    '''
    ncbitax_example = '''ncbitax example:

    #update ncbi taxonomy database:
    ncbitax -u
    '''
    of2orthomap_example = '''of2orthomap example:

    # download OrthoFinder example:
    $ wget https://zenodo.org/record/7796253/files/ensembl_105_orthofinder_Orthogroups.GeneCount.tsv.zip
    $ wget https://zenodo.org/record/7796253/files/ensembl_105_orthofinder_Orthogroups.tsv.zip
    $ wget https://zenodo.org/record/7796253/files/ensembl_105_orthofinder_species_list.tsv

    # extract orthomap:
    $ of2orthomap -seqname Danio_rerio.GRCz11.cds.longest -qt 7955 \\
      -sl ensembl_105_orthofinder_species_list.tsv \\
      -oc ensembl_105_orthofinder_Orthogroups.GeneCount.tsv.zip \\
      -og ensembl_105_orthofinder_Orthogroups.tsv.zip
    '''
    plaza2orthomap_example = '''plaza2orthomap example:
    
    # using Orthologous gene family 
    $ plaza2orthomap -qt 3702 \\
      -sl species_information.csv \\
      -og genefamily_data.ORTHOFAM.csv \\
      -out 3702.orthofam.orthomap
    
    # using Homologous gene family 
    $ plaza2orthomap -qt 3702 \\
      -sl species_information.csv \\ 
      -og genefamily_data.HOMFAM.csv \\
      -out 3702.homfam.orthomap
    '''
    qlin_example = '''qlin example:

    # get query lineage to be used with orthomap later on using query species taxid
    # Mus musculus; 10090
    $ qlin -qt 10090

    # using query species name
    $ qlin -q "Mus musculus"
    '''
    cds2aa_parser = subparsers.add_parser(name='cds2aa',
                                          help='translate CDS to AA and optional retain longest isoform <cds2aa -h>',
                                          epilog=cds2aa_example,
                                          formatter_class=argparse.RawDescriptionHelpFormatter)
    gtf2t2g_parser = subparsers.add_parser(name='gtf2t2g',
                                           help='extract transcript to gene table from GTF <gtf2t2g -h>',
                                           epilog=gtf2t2g_example,
                                           formatter_class=argparse.RawDescriptionHelpFormatter)
    ncbitax_parser = subparsers.add_parser(name='ncbitax',
                                           help='update local ncbi taxonomy database <ncbitax -h>',
                                           epilog=ncbitax_example,
                                           formatter_class=argparse.RawDescriptionHelpFormatter)
    of2orthomap_parser = subparsers.add_parser(name='of2orthomap',
                                               help='extract orthomap from OrthoFinder output for query species '
                                                    '<of2orthomap -h>',
                                               epilog=of2orthomap_example,
                                               formatter_class=argparse.RawDescriptionHelpFormatter)
    plaza2orthomap_parser = subparsers.add_parser(name='plaza2orthomap',
                                                  help='extract orthomap from PLAZA gene family data for query species '
                                                       '<of2orthomap -h>',
                                                  epilog=plaza2orthomap_example,
                                                  formatter_class=argparse.RawDescriptionHelpFormatter)
    qlin_parser = subparsers.add_parser(name='qlin',
                                        help='get query lineage based on ncbi taxonomy <qlin -h>',
                                        epilog=qlin_example,
                                        formatter_class=argparse.RawDescriptionHelpFormatter)
    cds2aa.add_argparse_args(parser=cds2aa_parser)
    gtf2t2g.add_argparse_args(parser=gtf2t2g_parser)
    ncbitax.add_argparse_args(parser=ncbitax_parser)
    of2orthomap.add_argparse_args(parser=of2orthomap_parser)
    plaza2orthomap.add_argparse_args(parser=plaza2orthomap_parser)
    qlin.add_argparse_args(parser=qlin_parser)
    return parser


def main():
    """
    The main function that is being called when `orthomap` is used via the terminal.
    """
    parser = define_parser()
    args = parser.parse_args()
    if args.subcommand is None:
        parser.print_help()
        sys.exit()
    if args.subcommand == 'cds2aa':
        if args.o is None:
            sys.stderr.write(str(args))
        else:
            print(args)
        cds2aa.cds2aa_fasta(args,
                            parser)
    if args.subcommand == 'gtf2t2g':
        print(args)
        if not args.i:
            parser.print_help()
            print('\nError <-i>: Please specify GTF input file')
            sys.exit()
        if args.o:
            if os.path.exists(args.o) and not args.overwrite:
                print('\nError <-overwrite>: output file exists, please set to True if it should be overwritten\n')
                sys.exit()
            output = open(args.o,
                          'w')
        else:
            output = sys.stdout
        gtf2t2g.parse_gtf(gtf=args.i,
                          g=args.g,
                          b=args.b,
                          p=args.p,
                          v=args.v,
                          s=args.s,
                          output=output,
                          q=args.q)
        output.close()
    if args.subcommand == 'ncbitax':
        print(args)
        if not args.u:
            parser.print_help()
            print('\nError <-u>: Please specify if you like to update <-u>')
            sys.exit()
        if args.u:
            ncbitax.update_ncbi()
    if args.subcommand == 'of2orthomap':
        print(args)
        if not args.seqname:
            parser.print_help()
            print('\nError <-seqname>: Please specify query species name in orthofinder and taxid')
            sys.exit()
        if not args.qt:
            parser.print_help()
            print('\nError <-qt>: Please specify query species taxid')
            sys.exit()
        if not args.sl:
            parser.print_help()
            print('\nError <-sl>: Please specify species list as <orthofinder name><tab><species taxid>')
            sys.exit()
        if not args.oc:
            parser.print_help()
            print('\nError <-oc>: Please specify orthofinder <Orthogroups.GeneCounts.tsv> (see Orthogroups directory)')
            sys.exit()
        if not args.og:
            parser.print_help()
            print('\nError <-og>: Please specify orthofinder <Orthogroups.tsv> (see Orthogroups directory)')
            sys.exit()
        of2orthomap.get_orthomap(seqname=args.seqname,
                                 qt=args.qt,
                                 sl=args.sl,
                                 oc=args.oc,
                                 og=args.og,
                                 out=args.out,
                                 quiet=False,
                                 continuity=True,
                                 overwrite=args.overwrite)
    if args.subcommand == 'plaza2orthomap':
        print(args)
        if not args.qt:
            parser.print_help()
            print('\nError <-qt>: Please specify query species taxID')
            sys.exit()
        if not args.sl:
            parser.print_help()
            print('\nError <-sl>: Please specify PLAZA species information file <species_information.csv>')
            sys.exit()
        if not args.og:
            parser.print_help()
            print('\nError <-og>: Please specify PLAZA gene family file <genefamily_data.ORTHOFAM.csv> or '
                  '<genefamily_data.HOMFAM.csv>')
            sys.exit()
        plaza2orthomap.get_plaza_orthomap(qt=args.qt,
                                          sl=args.sl,
                                          og=args.og,
                                          out=args.out,
                                          quiet=False,
                                          continuity=True,
                                          overwrite=args.overwrite)
    if args.subcommand == 'qlin':
        print(args)
        if not args.q and not args.qt:
            parser.print_help()
            print('\nError <-q> <-qt>: Please specify query species name or taxid')
            sys.exit()
        if args.q and args.qt:
            parser.print_help()
            print('\nWarning: Since both query species name and taxid are given taxid is used')
            sys.exit()
        qlin.get_qlin(q=args.q,
                      qt=args.qt,
                      quiet=False)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: UTF-8 -*-


import pandas as pd
from orthomap import datasets, orthomap2tei


file = datasets.sun21_orthomap(datapath='/tmp')


def test_read_orthomap():
    expected_columns = ['GeneID', 'Phylostratum']
    query_orthomap = orthomap2tei.read_orthomap(file)
    assert isinstance(query_orthomap, pd.DataFrame)
    assert all(query_orthomap.columns == expected_columns)


def test_geneset_overlap():
    g1 = ['A', 'B', 'C']
    g2 = ['A', 'D', 'E', 'F']
    overlap = orthomap2tei.geneset_overlap(g1, g2)
    assert overlap['g1_g2_overlap'].values == 1
    assert overlap['g1_ratio'].values == 1 / 3
    assert overlap['g2_ratio'].values == 1 / 4


def test_replace_by():
    # Note that with this function you always have to replace all the elements.
    x = ['A', 'B', 'C']
    excepted_x = ['D', 'K', 'E']
    xmatch = ['A', 'C', 'B']
    xreplace = ['D', 'E', 'K']
    newx = orthomap2tei.replace_by(x, xmatch, xreplace)
    assert newx == excepted_x


def test_keep_min_max_case_max():
    # Not sure we need a function for this. It can be done in two lines.
    my_orthomap = pd.DataFrame.from_dict(
        {'GeneID': ['g1', 'g1', 'g2', 'g3', 'g3'], 'Phylostrata': [3, 1, 2, 5, 7]}
    )
    ascending = {'max': False, 'min': True}
    expected_orthomap = (
        my_orthomap.sort_values('Phylostrata', ascending=ascending['max'])
        .drop_duplicates('GeneID')
        .sort_values('Phylostrata', ascending=ascending['max'])
    )
    filtered_orthomap = orthomap2tei._keep_min_max(my_orthomap, keep='max')
    assert (expected_orthomap.values == filtered_orthomap.values).all()


def test_keep_min_max_case_min():
    # Not sure we need a function for this. It can be done in two lines.
    my_orthomap = pd.DataFrame.from_dict(
        {'GeneID': ['g1', 'g1', 'g2', 'g3', 'g3'], 'Phylostrata': [3, 1, 2, 5, 7]}
    )
    ascending = {'max': False, 'min': True}
    expected_orthomap = (
        my_orthomap.sort_values('Phylostrata', ascending=ascending['min'])
        .drop_duplicates('GeneID')
        .sort_values('Phylostrata', ascending=ascending['min'])
    )
    filtered_orthomap = orthomap2tei._keep_min_max(my_orthomap, keep='min')
    assert (expected_orthomap.values == filtered_orthomap.values).all()


def test_split_gene_id_by_gene_age():
    # Not sure what the function is doing.
    pass


def test_get_psd():
    # No data to run the example.
    pass


def test_get_pmatrix():
    pass


def test_get_tei():
    pass


def test_get_pstrata():
    pass


def test_min_max_to_01():
    ndarray_example_one = [1 for _ in range(5)]
    assert orthomap2tei._min_max_to_01(ndarray_example_one) == [0 for _ in range(5)]
    ndarray_example_two = [1 for _ in range(5)] + [2]
    assert orthomap2tei._min_max_to_01(ndarray_example_two) == [0 for _ in range(5)] + [1]



def test_get_ematrix():
    pass


def test_get_rematrix():
    pass
